/*
 * FILE:    sip_mbus_control.c
 * PROGRAM: SIP-MBUS - controller
 * AUTHOR:  KyungTae Kim
 * VERSION: 1.0
 *
 * This is the main program for the RAT controller for the SIPC.  
 *
 * Copyright (c) 2001 Columbia University 
 * All rights reserved.
 */

#include "sip_mbus_control.h"
#include "config_unix.h"
#include "config_win32.h"
#include "debug.h"
#include "version.h"
#include "crypt_random.h"
#include "mbus.h"
#include "mbus_parser.h"
#include "net_udp.h"
#include "util.h"

#include <errno.h>
#include <limits.h>
#include <signal.h>
#include <pthread.h>
#include <unistd.h>

#define DEFAULT_PORT 30521

static void *start_func(void *arg);

char       *u_addr, *e_addr[2];
int         should_exit;

static void mbus_err_handler(int seqnum, int reason) {
  if (should_exit == FALSE) {
#ifdef DEBUG
    fprintf(stderr, "Could not send mbus message (%d:%d)\n", seqnum, reason);
#endif
    abort();
  }
}

void sip_mbus_control() {
  struct mbus *m;
  char        c_addr[60];
  int	      seed = (gethostid() << 8) | (getpid() & 0xff);

  int ret;

  srand48(seed);

  /* create the mbus address of sip-mbus controller */
  snprintf(c_addr, 60, "(media:audio module:control app:sip id:%lu)", (unsigned long) getpid());

  /* initialization of mbus library */
  m = mbus_init(mbus_control_rx, mbus_err_handler, c_addr);
  if (m == NULL) {
#ifdef DEBUG
    fprintf("error in mbus_init\n");
#endif
    abort();
  }

  /* default behavior */
  ret = pthread_create(NULL, NULL, &start_func, (void *)m);
  if(ret < 0) {
#ifdef DEBUG
    fprintf("error in spawning a thread\n");
#endif
    abort();
  }
}

void *start_func(void *arg) {

  struct mbus *m;
  struct timeval	 timeout;
  int final_iters = 250;

  pthread_detach(pthread_self());

  m = (struct mbus *)arg;

#ifdef DEBUG
  should_exit = TRUE;
#endif
  while (final_iters > 0) {
#ifdef DEBUG
    fprintf(stderr, "sending mbus.hello\n");
#endif
	/* send mbus.hello command */
    mbus_send(m);
    mbus_heartbeat(m, 1);
    mbus_retransmit(m);
    timeout.tv_sec  = 0;
    timeout.tv_usec = 100000;

	/* mbus handler process the received mbus command */
    mbus_recv(m, m, &timeout);
	if (should_exit) {
	  final_iters--;
	}
  }        
#ifdef DEBUG
  fprintf(stderr, "For testing, after sending mbus.hello 250 times, exits\n");
#endif
}
